@@ -7,6 +7,9 @@
 ;; Created: 27 Dec 2007
 ;; Version: 0.01
 ;; Keywords: abbrev
+;; 
+;; This file is part of PDE (Perl Development Environment).
+;; But it is useful for generic programming.
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -86,6 +89,10 @@
 ;;        (replace-regexp-in-string "['\"]?\\(.*\\)['\"]" "\\1" file))
 ;;     ": $!\";")))
 
+;;; Dependencies:
+;;  no extra libraries is required
+
+;;; Installation:
 ;; Put this file into your load-path and the following into your ~/.emacs:
 ;;   (require 'tempo-x)
 
@@ -98,7 +105,8 @@
 (defgroup tempo-x nil
   "Various tempo elements handler."
   :group 'abbrev
-  :group 'convenience)
+  :group 'convenience
+  :group 'pde)
 
 ;;;###autoload 
 (defun tempo-x-space ()
@@ -140,6 +148,7 @@ Like `tempo-insert-prompt', but you can give parameters for
            "")
           (saved "")
           (tempo-interactive
+           (if (symbolp default) (setq default (symbol-value default)))
            (cond ((stringp prompt)
                   (setq insertion (read-from-minibuffer prompt default)))
                  ((and (listp prompt) (stringp (car prompt)))
@@ -229,8 +238,10 @@ FIXME: the regexp above is display incorrect.
 ;;; 1. use with other tempo elements as well as users elements
 ;;; 2. add keymap in snippet region
 ;;; 3. easy interface for form.
-(defvar tempo-x-exclude-chars ""
-  "*Chars not to extend when add to front or end of the field.")
+(defcustom tempo-x-exclude-chars ""
+  "*Chars not to extend when add to front or end of the field."
+  :type 'string
+  :group 'tempo-x)
 
 (defface tempo-x-editable-face
   '((((background dark)) (:background "steel blue"))
@@ -376,10 +387,12 @@ changed, the text of form will change too."
     (mapc (lambda (name)
             (let ((source (tempo-x-find-source name))
                   forms)
-              (when source
-                (setq forms (overlay-get source 'tempo-x-forms))
-                (push ov forms)
-                (overlay-put source 'tempo-x-forms forms))))
+              (if source
+                  (progn
+                    (setq forms (overlay-get source 'tempo-x-forms))
+                    (push ov forms)
+                    (overlay-put source 'tempo-x-forms forms))
+                (error "Unknown form variable '%s': maybe you didn't source it yet!" name))))
           vars)
     (push ov tempo-x-snippet-forms)))
 
@@ -425,6 +438,10 @@ changed, the text of form will change too."
   "Clear OVERLAY and its mirrors."
   (mapc 'tempo-x-delete-overlay
         (overlay-get overlay 'tempo-x-mirrors))
+  (mapc (lambda (ov)
+          (setq tempo-x-snippet-forms (delq ov tempo-x-snippet-forms))
+          (tempo-x-delete-overlay ov))
+        (overlay-get overlay 'tempo-x-forms))
   (setq tempo-x-snippet-sources
         (delq (assq (overlay-get overlay 'tempo-x-name) tempo-x-snippet-sources)
               tempo-x-snippet-sources))
